@@ -72,10 +72,13 @@ This module generates ANSI character codes to printing colors to terminals.
 import os
 import re
 from abc import ABC
-from typing import List, Optional, Pattern
+from collections import deque
+from functools import partial
+from typing import Deque, Iterable, List, Optional, Pattern, Type, TypeVar, Union
 
 # 3rd party
 import click
+from domdf_python_tools.iterative import chunks
 from typing_extensions import Final
 
 __all__ = [
@@ -104,9 +107,12 @@ try:
 	# 3rd party
 	import colorama  # type: ignore
 	colorama.init()
+
 except ImportError:
 	pass
 
+_C = TypeVar("_C", bound="Colour")
+
 ColourTrilean = Optional[bool]
 """
 Represents the :py:obj:`True`/:py:obj:`False`/:py:obj:`None` state of colour options.
@@ -118,9 +124,9 @@ CSI: Final[str] = "\u001b["
 OSC: Final[str] = "\u001b]"
 BEL: Final[str] = '\x07'
 
-fore_stack: List[str] = []
-back_stack: List[str] = []
-style_stack: List[str] = []
+fore_stack: Deque[str] = deque()
+back_stack: Deque[str] = deque()
+style_stack: Deque[str] = deque()
 
 
 def resolve_color_default(color: ColourTrilean = None) -> ColourTrilean:
@@ -174,8 +180,6 @@ def strip_ansi(value: str) -> str:
 	"""
 	Strip ANSI colour codes from the given string to return a plaintext output.
 
-	.. versionadded:: 1.1.0
-
 	:param value:
 	"""
 
@@ -196,17 +200,19 @@ class Colour(str):
 	:type reset: :class:`str`
 	"""
 
+	__slots__ = ("style", "reset", "stack")
+
 	style: str
 	reset: str
-	stack: List[str]
+	stack: Union[Deque[str], List[str]]
 
-	def __new__(cls, style: str, stack: List[str], reset: str) -> "Colour":  # noqa D102
-		color = super().__new__(cls, style)  # type: ignore
-		color.style = style
-		color.stack = stack
-		color.reset = reset
+	def __new__(cls, style: str, stack: Union[Deque[str], List[str]], reset: str) -> "Colour":  # noqa D102
+		self = super().__new__(cls, style)  # type: ignore
+		self.style = style
+		self.stack = stack
+		self.reset = reset
 
-		return color
+		return self
 
 	def __enter__(self) -> None:
 		print(self.style, end='')
@@ -224,13 +230,177 @@ class Colour(str):
 
 		return f"{self}{text}{self.reset}"
 
+	@classmethod
+	def from_code(cls: Type[_C], code: Union[str, int], background: bool = False) -> _C:
+		"""
+		Returns a :class:`~.Colour` to create coloured text.
+
+		The colour can be reset using :py:obj:`.Fore.RESET` or :py:obj:`.Back.RESET`.
+
+		.. versionadded:: 0.9.0
+
+		:param code: A 3- or 4- bit ANSI colour code.
+		:param background: Whether to set the colour for the background.
+
+		:rtype:
+
+		.. note::
+
+			The ``background`` option only influences the reset value and the stack used.
+			It will not handle conversion of foreground codes to background codes etc.
+		"""
+
+		if background:
+			stack = back_stack
+			reset = AnsiBack._reset
+		else:
+			stack = fore_stack
+			reset = AnsiFore._reset
+
+		return cls(code_to_chars(code), stack=stack, reset=reset)
+
+	@classmethod
+	def from_256_code(cls: Type[_C], code: Union[str, int], background: bool = False) -> _C:
+		"""
+		Returns a :class:`~.Colour` to create 256-colour text.
+
+		The colour can be reset using :py:obj:`.Fore.RESET` or :py:obj:`.Back.RESET`.
+
+		.. versionadded:: 0.9.0
+
+		.. note:: Not all terminals support 256-colour mode.
+
+		:param code: A 256-colour ANSI code.
+		:param background: Whether to set the colour for the background.
+		"""
+
+		if background:
+			stack = back_stack
+			reset = AnsiBack._reset
+			template = CSI + "48;5;{}m"
+
+		else:
+			stack = fore_stack
+			reset = AnsiFore._reset
+			template = CSI + "38;5;{}m"
+
+		return cls(template.format(code), stack=stack, reset=reset)
+
+	@classmethod
+	def from_rgb(
+			cls: Type[_C],
+			r: Union[str, int],
+			g: Union[str, int],
+			b: Union[str, int],
+			background: bool = False
+			) -> _C:
+		"""
+		Returns a :class:`~.Colour` to create 24-bit coloured text.
+
+		The colour can be reset using :py:obj:`.Fore.RESET` or :py:obj:`.Back.RESET`.
+
+		.. versionadded:: 0.9.0
+
+		.. note:: Not all terminals support 24-bit colours.
+
+		:param r:
+		:param g:
+		:param b:
+		:param background: Whether to set the colour for the background.
+		"""
+
+		if background:
+			template = CSI + "48;2;{r};{g};{b}m"
+			stack = back_stack
+			reset = AnsiBack._reset
+
+		else:
+			template = CSI + "38;2;{r};{g};{b}m"
+			stack = fore_stack
+			reset = AnsiFore._reset
+
+		return cls(template.format(r=r, g=g, b=b), stack=stack, reset=reset)
+
+	@classmethod
+	def from_hex(cls: Type[_C], hex_colour: str, background: bool = False) -> _C:
+		"""
+		Returns a :class:`~.Colour` to create 24-bit coloured text.
+
+		The colour can be reset using :py:obj:`.Fore.RESET` or :py:obj:`.Back.RESET`.
+
+		.. versionadded:: 0.9.0
+
+		.. note:: Not all terminals support 24-bit colours.
+
+		:param hex_colour: The hex colour code.
+		:param background: Whether to set the colour for the background.
+		"""
+
+		# From https://stackoverflow.com/q/29643352
+		# https://stackoverflow.com/users/3924370/julian-white
+		# CC BY-SA 3.0
+
+		value = hex_colour.lstrip('#')
+		lv = len(value)
+		r, g, b = tuple(int(value[i:i + lv // 3], 16) for i in range(0, lv, lv // 3))
+
+		return cls.from_rgb(r, g, b, background=background)
+
+
+def print_256_colour_testpattern() -> None:
+	"""
+	Print a 256-colour test pattern to the terminal.
+
+	.. versionadded:: 0.9.0
+
+	.. note:: Not all terminals support 24-bit colours.
+	"""
+
+	def print_heading(text: str, block_size: int = 3, n_blocks: int = 36):
+		click.echo()
+		click.echo(str(text).center(((block_size + 1) * n_blocks) - 1, '-'))
+
+	def print_line(values: Iterable[int], block_size: int = 3):
+		values = list(values)
+		mid = len(values) // 2
+
+		colour: Optional[bool] = resolve_color_default()
+		echo = partial(click.echo, nl=False, color=colour)
+
+		for code in values[:mid]:
+			echo(Colour.from_256_code(code, background=True)(str(code).center(block_size)) + ' ')
+		with Fore.BLACK:
+			for code in values[mid:-1]:
+				echo(Colour.from_256_code(code, background=True)(str(code).center(block_size)) + ' ')
+			click.echo(
+					Colour.from_256_code(values[-1], background=True)(str(values[-1]).center(block_size).rstrip()),
+					color=colour
+					)
+
+	print_heading(
+			"Standard Colours".center((9 * 8) - 1, '-') + ' ' + "High-Intensity Colours".center((9 * 8) - 1, '-'),
+			block_size=8,
+			n_blocks=16
+			)
+
+	print_line(range(16), block_size=8)
+
+	print_heading("216 Colours")
+	for row in chunks(range(16, 232), 36):
+		print_line(row)
+
+	print_heading("Greyscale Colours", block_size=5, n_blocks=24)
+	print_line(range(232, 256), block_size=5)
+
+	click.echo('\n')
+
 
 class AnsiCodes(ABC):
 	"""
 	Abstract base class for ANSI Codes.
 	"""
 
-	_stack: List[str]
+	_stack: Union[Deque[str], List[str]]
 	_reset: str
 
 	def __init__(self) -> None:
@@ -305,7 +475,7 @@ class AnsiCursor:
 		.. versionadded:: 0.7.0
 		"""
 
-		return "\u001b[?25l"
+		return f"{CSI}?25l"
 
 	def SHOW(self) -> str:
 		"""
@@ -314,7 +484,7 @@ class AnsiCursor:
 		.. versionadded:: 0.7.0
 		"""
 
-		return "\u001b[?25h"
+		return f"{CSI}?25h"
 
 
 class AnsiFore(AnsiCodes):
@@ -345,7 +515,7 @@ class AnsiFore(AnsiCodes):
 	"""
 
 	_stack = fore_stack
-	_reset = "\u001b[39m"
+	_reset = f"{CSI}39m"
 
 	BLACK = 30
 	RED = 31
@@ -396,7 +566,7 @@ class AnsiBack(AnsiCodes):
 	"""
 
 	_stack = back_stack
-	_reset = "\u001b[49m"
+	_reset = f"{CSI}49m"
 
 	BLACK = 40
 	RED = 41
@@ -434,7 +604,7 @@ class AnsiStyle(AnsiCodes):
 	"""
 
 	_stack = style_stack
-	_reset = "\u001b[22m"
+	_reset = f"{CSI}22m"
 
 	BRIGHT = 1
 	DIM = 2
@@ -450,3 +620,6 @@ Cursor = AnsiCursor()
 fore_stack.append(Fore.RESET)
 back_stack.append(Back.RESET)
 style_stack.append(Style.NORMAL)
+
+if __name__ == "__main__":
+	print_256_colour_testpattern()
